/*************************************************************************
 *
 *	GUI's low level variables
 *
 *	Caution: these variables will never be used by any external function
 *
 *************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <linux/fb.h>
#include <linux/gui.h>
#include <linux/gui_internal.h>

#include "mini_gui.h"
#include "fill_patterns.h"


static char * device = "/dev/fb0";
static struct fb_var_screeninfo screeninfo;
static int screen_fd;

static char *dev_digi = "/dev/digi";
static char *dev_kb = "/dev/kb";
static char *dev_serial = "/dev/serial";
static char *dev_peri = "/dev/peri";
static char *dev_gui = "/dev/gui";
static char *dev_irda = "/dev/irda";
static char *dev_mini_db = "/dev/mini_db";

int	digi_fd;
int  kb_fd;
int  serial_fd;
int  peri_fd;
int  gui_fd;
int	irda_fd;
int	mini_db_fd;

/*---------------------------
  to record the left-top corner of current FORM,
  COZ the components in the FORM uses relative co-ordinates
----------------------------*/
short xBase,yBase; 


U8 * screen_ptr;
short screen_width;
short screen_height;

// the current available region 
RECT av_region;

/***********************************************************
 *
 *	GUI low level functions
 *
 *	Mac Tseng     11/2/2000,   LinuxDA Co.,Ltd 
 *
 **********************************************************/

void close_screen(void)
{
	close(screen_fd);
	close(digi_fd);
	//close(kb_fd);
	close(peri_fd);
	close(gui_fd);
	close(mini_db_fd);
}

/**  initialize the screen ptr and the digitizer ptr at the same time ******/
void init_screen(void)
{
	short code;
	
	peri_fd = open(dev_peri,O_RDWR);
	if (peri_fd == -1) {
		perror("Unable to open peri device driver");
		exit(0);
	}

	gui_fd = open(dev_gui,O_RDWR);
	if (gui_fd == -1) {
		perror("Unable to open gui device driver");
		exit(0);
	}
	mini_db_fd = open(dev_mini_db,O_RDWR);
	if (mini_db_fd == -1) {
		perror("Unable to open mini db device driver");
		exit(0);
	}

	screen_fd = open(device, O_RDWR);
	if (screen_fd == -1) {
		perror("Unable to open frame buffer device /dev/fb0");
		exit(0);
	}

	digi_fd = open(dev_digi,O_RDONLY);
	if (digi_fd == -1) {
		perror("Unable to open digitizer device driver");
		exit(0);
	}
	
	//kb_fd = open(dev_kb,O_RDONLY);
	//if (kb_fd == -1) {
	//	perror("Unable to open keyboard device driver");
		//exit(0);
	//}
	RegisterChFonts();
		
	
	
	if (ioctl(screen_fd, FBIOGET_VSCREENINFO, &screeninfo)==-1) {
		perror("Unable to retrieve framebuffer information");
		exit(0);
	}
	screen_width = screeninfo.xres_virtual;
	screen_height = screeninfo.yres_virtual;
	
	screen_ptr = mmap(0, screen_height * screen_width / 8, PROT_READ|PROT_WRITE, 0, screen_fd, 0);
	
	//set the available region as full screen
	av_region.left = av_region.top = 0;
	av_region.right = screen_width;
	av_region.bottom = screen_height;
	
}
U16 GetBatteryPower(void)
{
	U16 bty;
	U16 retValue;

	short code=CMD_GET_BATTERY;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&bty,sizeof(bty));

	retValue = bty<100?bty:100;
	
	return retValue;
}
void HardReset(void)
{
	short code=CMD_HARD_RESET;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));

}
void ShowMem(void)
{
	short code=CMD_SHOW_MEM;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}

void SetAutoOffTime(short t)
{
	short code=CMD_SET_AUTO_OFF;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(code));
}
short GetAutoOffTime(void)
{
	short code=CMD_GET_AUTO_OFF;
	// send the set contrast command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
	
	return code;
}
/**** system beep vol ****/
void SetSysBeepVol(short t)
{
	short code=CMD_SET_SYS_BEEP_VOL;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(t));
}
short GetSysBeepVol(void)
{
	short t;
	short code=CMD_GET_SYS_BEEP_VOL;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(t));
	
	return t;
}
void SysBeep(void)
{	
	short code=CMD_SYS_BEEP;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
void GameBeep(void)
{
	short code=CMD_GAME_BEEP;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
void AlarmBeep(void)
{
	short code=CMD_ALARM_BEEP;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
/**** alram beep vol ****/
void SetAlarmBeepVol(short t)
{
	short code=CMD_SET_ALARM_BEEP_VOL;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(t));
}
short GetAlarmBeepVol(void)
{
	short t;
	short code=CMD_GET_ALARM_BEEP_VOL;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(t));
	
	return t;
}

/**** game beep vol ****/
void SetGameBeepVol(short t)
{
	short code=CMD_SET_GAME_BEEP_VOL;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(code));
}
short GetGameBeepVol(void)
{
	short t;
	short code=CMD_GET_GAME_BEEP_VOL;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&t,sizeof(t));
	
	return t;
}

/*** Soft keyboard type *****/
short SetKeyboardType(short k_type)
{
	short code=CMD_SET_KB_TYPE;
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&k_type,sizeof(k_type));
}


void SuspendMode(void)
{
	short code=CMD_SUSPEND_MODE;
	// send the set contrast command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
void PowerOff(void)
{
	short code=CMD_POWER_OFF;
	// send the set contrast command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
	//printf("Poweroff\n");
}	
void SetContrast(short contrast)
{
	short code=CMD_SET_CONTRAST;
	// send the set contrast command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&contrast,sizeof(code));	
}
void SwitchBackLight(void)
{
	short code=CMD_SET_BACKLIGHT;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
// to check if the battery in charging, return 1 if yes, otherwise 0 will be return
short InCharging(void)
{
	short ret_value;
	short code=CMD_IN_CHARGING;
	// send the set contrast command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&ret_value,sizeof(ret_value));
	
	return ret_value;
}	
/************ IRDA related functions ********/
void OpenIRDA(void)
{
	irda_fd = open(dev_irda,O_RDWR);
}
void CloseIRDA(void)
{
	close(irda_fd);
}
short ReadIRDA(U8 *data,short s_len)
{
	short ret_value;
	ret_value = read(irda_fd,data,s_len);
	return ret_value;
}
short WriteIRDA(U8 *code,short s_len)
{
	short ret_value;
	
	ret_value = write(irda_fd,code,s_len);
	return ret_value;
}

/************ UART related functions ********/
void OpenUART(void)
{
		serial_fd = open(dev_serial,O_RDWR);
}
void CloseUART(void)
{
	close(serial_fd);
}
// return 0 if success, return -1 if timed out
short WriteUART(U8 code)
{
	short ret_value;
	
	ret_value = write(serial_fd,&code,sizeof(code));
	return ret_value;
}

// return 0 if success, return -1 if timed out
short ReadUART(U8 *data)
{
	short ret_value;
	ret_value = read(serial_fd,data,sizeof(*data));
	return ret_value;
}
void SetBaudRate(long brate)
{
	//av_region = *r;
	CMD_GUI code = CMD_SET_BAUD_RATE;
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&brate,sizeof(brate));
}


void Beep(U16 rate)
{	
	short code=CMD_WRITE_SPEAKER;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&rate,sizeof(rate));
}
short IsRamDiskLost(void)
{
	short code=CMD_RAMDISK_LOST;
	short ret_value;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&ret_value,sizeof(ret_value));
	return ret_value;
}

// Sets digitizer data type
void SetDigitizerDataType(short d_type)
{
	short code=CMD_SET_DIGITIZER_DATA_TYPE;
	// send the set digitizer data type command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&d_type,sizeof(d_type));
}
// any pixel out of this region will be bypass!
void SetAvailableRegion(RECT *r)
{
	//av_region = *r;
	CMD_GUI code = CMD_SET_AVAILABLE_REGION;
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,r,sizeof(r));	
}
void ClipRect(RECT *r)
{
	CMD_GUI code=CMD_CLIPRECT;
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,r,sizeof(r));
}
void xor_draw_line(U16 x1, U16 y1, U16 x2, U16 y2)
{
	CMD_GUI code=CMD_XOR_DRAW_LINE;
	ARG_XOR_DRAW_LINE arg={x1,y1,x2,y2};
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}

void draw_line(short x1, short y1, short x2, short y2, short color)
{
	CMD_GUI code=CMD_DRAW_LINE;
	ARG_DRAW_LINE arg={x1,y1,x2,y2,color};
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}


void draw_pixel(short x, short y, U8 color)
{
	CMD_GUI code=CMD_DRAW_PIXEL;
	ARG_DRAW_PIXEL arg={x,y,color};
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}

void draw_ellipse(short x, short y, short a, short b, GraphicsContext *gc)
{
	CMD_GUI code=CMD_DRAW_ELLIPSE;
	ARG_DRAW_ELLIPSE arg={x,y,a,b,gc};
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}

void display_bitmap(U8 * bitmap_p)
{
	memcpy(screen_ptr ,bitmap_p , screen_width * screen_height/8);
}

void bitblt(short src_x,short src_y,short w,short h,short dest_x,short dest_y,U8 *src,short src_units_per_line, 
		 U8 *dest,short dest_units_per_line,GraphicsContext *gc)
{
	CMD_GUI code=CMD_BITBLT;
	ARG_BITBLT arg={src_x,src_y,w,h,dest_x,dest_y,src,src_units_per_line,dest,dest_units_per_line,gc};
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}

void patternfill(short dest_x,short dest_y, short w,short h,U8 *dest,
			short dest_units_per_line,GraphicsContext *gc)
{
	CMD_GUI code=CMD_PATTERNFILL;
	ARG_PATTERNFILL arg={dest_x,dest_y,w,h,dest,dest_units_per_line,gc};
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));  
}

void FillRect(RECT r,GraphicsContext *gc)
{
	ClipRect(&r);
	patternfill(r.left, r.top, r.right-r.left, r.bottom-r.top, screen_ptr, 20, gc);
}
void draw_rect(RECT r,GraphicsContext *gc)
{
	CMD_GUI code=CMD_DRAW_RECT;
	ARG_DRAW_RECT arg={r,gc};
	// send the set contrast command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}

/* Copy the screen information to memory, but this function will NOT assign 
 * any memory spaces
 */
void CopyScreen(U8 *p,U16 x,U16 y,U16 w,U16 h)
{
	U16 i,bytes_per_line,x1;
	U8 *tp;
	
	x1 = x + w;x = x/8;
	tp = p;bytes_per_line = x1/8 -x + 1;
	for (i = 0;i <= h;i++) {
		memcpy(tp,screen_ptr+((i+y)*screen_width)/8 + x,bytes_per_line);
		tp += bytes_per_line;
	}
}

void PasteScreen(U8 *p,U16 x,U16 y,U16 w,U16 h)
{
	U16 i,bytes_per_line,x1;
	U8 *tp;

	x1 = x +w;x = x/8;
	tp = p;bytes_per_line = x1/8 - x + 1;
	for (i = 0;i <= h;i++) {
		memcpy(screen_ptr+((i+y)*screen_width)/8 + x,tp,bytes_per_line);
		tp += bytes_per_line;
	}
	
}

U8 *MallocScreen(U16 w,U16 h)
{
	U16 dlg_size,i;
	dlg_size = (w/8+1)*8 * h/8;

	return malloc(dlg_size);
}
U8 FreeScreen(U8 *p)
{
	return free(p);
}

U8 PixelsUnits(short pxs)
{
	if (pxs > 0)
		return ((pxs-1)/8+1);
	else
		return 0;
}
/*************************************************
 *
 *	Using the dot-line to script a rectangle
 *
 *************************************************/
void DrawDotRect(RECT r)
{
	RECT r1;
	GraphicsContext gc;

	gc.mode = Mode_SRC;gc.fill_index = HoriDotPattern;
	r1 = r;
	r1.bottom = r.top+1;							// top line
	FillRect(r1,&gc);
	r1.top = r.bottom-1;r1.bottom = r.bottom;		// bottom
	FillRect(r1,&gc);

	gc.mode = Mode_SRC;gc.fill_index = VertDotPattern;
	r1 = r; r1.right = r.left + 1;					// left
	FillRect(r1,&gc);
	r1 = r; r1.left = r.right - 1;					// right
	FillRect(r1,&gc);

}

U8 PixelInRect(PIXEL px,RECT r)
{
	if ((px.x >= r.left) && (px.x <= r.right) && (px.y >= r.top) &&(px.y <= r.bottom)) {
		return 1;
	}
	else {
		return 0;
	}
}

U16 lobyte(U32 lParam)
{
	return (U16)lParam & 0xffff;
}

U16 hibyte(U32 lParam)
{
	return (U16)((lParam&0xffff0000)>>16);
}

U8 *GetScreenPtr(void)
{
	return screen_ptr;
}

U16 GetScreenWidth(void)
{
	return screen_width;
}
U16 GetScreenHeight(void)
{
	return screen_height;
}
// run app as a child routine & hung up till it finish
void CallApp(char *app)
{
	int pid;
	pid = vfork();
	if (pid == 0) {	// the child routine
		execlp(app,app,NULL);
		//_exit(0);
	}
		else {
				waitpid(pid,NULL,0);
				//printf("Main win,comes from %s\n",app);
				//ShowMainWin();
		}
}

// show default style how-to form
short gui_how_to(char *str)
{
	short ret_value;
	CMD_GUI code=CMD_GUI_HOW_TO;
	ARG_GUI_HOW_TO arg={str,ret_value};
	
	// send the set contrast command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
	
	return arg.ret_value;
}


// show default style about form
short gui_about(char *title,char *version)
{
	short ret_value;
	CMD_GUI code=CMD_GUI_ABOUT;
	ARG_GUI_ABOUT arg={title,version,ret_value};
	
	// send the set contrast command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
	
	return arg.ret_value;
}

// Sets the calibration of digitizer
void SetCalibration(int xmin,int ymin,int xmax,int ymax)
{
	CMD_GUI code=CMD_SET_CALIBRATION;
	ARG_SET_CALIBRATION arg={xmin,ymin,xmax,ymax};
	
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
}
// access registers in directly
short SetRegister(U32 reg,U32 *data,short length)
{
	CMD_GUI code=CMD_SET_REGISTER;
	ARG_RW_REGISTER arg={reg,data,length};
	
	// send the set register command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
	return arg.ret_value;
}
short GetRegister(U32 reg,U32 *data,short length)
{
	CMD_GUI code=CMD_GET_REGISTER;
	ARG_RW_REGISTER arg={reg,data,length};
		
	// send the get register command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
	return arg.ret_value;
}
U32 SetCPUSpeed(U8 p,U8 q,U8 freq)
{
	CMD_GUI code=CMD_SET_CPU_SPEED;
	ARG_SET_CPU_SPEED arg={p,q,freq};
	
	// send the get register command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
	return arg.ret_value;
}
U32 GetCPUSpeed(U8 *p,U8 *q,U8 *freq)
{
	CMD_GUI code=CMD_GET_CPU_SPEED;
	ARG_GET_CPU_SPEED arg={p,q,freq};
	
	// send the get register command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&arg,sizeof(arg));
	return arg.ret_value;
}
short GetHandheldType(void)
{
	CMD_GUI code=CMD_GET_HANDHELD_TYPE;
	short ret_value;
	// send the get register command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&ret_value,sizeof(ret_value));
	
	return ret_value;
}
void SetLowestPriority(void)
{
	int tp;
	tp = getpid();
	setpriority(0,3,40);
	tp = getpriority(0,3);
}
/****** set current lowers priority & pause it to release the CPU resource *****/
void ResidentTask(void)
{
	SetLowestPriority();
	pause();
}
/********************* give back the launch command *********************************
 *	
 *	Due to every application can has different launch mode such as NORMAL,Get Info...etc
 *
 ************************************************************************************/
short GetLaunchCommand(int argc,char *argv[])
{
	char *s;
	
	if (argc < 1) return CMD_RUN_NORMAL;
	s=itoa(CMD_RUN_GET_INFO);
	if (strcmp(argv[0],s) == 0)
	{
		return CMD_RUN_GET_INFO;
	}
	else
		return CMD_RUN_NORMAL;
}
// copy app info structure to kernel
void PutAppInfo(APP_INFO *AppInfo)
{
	CMD_GUI code=CMD_PUT_APP_INFO;
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,AppInfo,sizeof(AppInfo));
}
void GetAppInfo(APP_INFO *AppInfo)
{
	CMD_GUI code = CMD_GET_APP_INFO;
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,AppInfo,sizeof(AppInfo));	
}

void SetCapsLock(void)
{
	CMD_GUI code = CMD_SET_CAPSLOCK;
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&code,sizeof(code));	
}

U8 GetCapsLock(void)
{
	CMD_GUI code = CMD_GET_CAPSLOCK;
	U8 ret_value;
	// send the command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&ret_value,sizeof(ret_value));	
	
	return ret_value;
}
// return 1 if file exist, otherwise 0 will be returned
static short FileExist(char *path_name)
{
	int fd;
	short ret_value;

	fd = open(path_name,O_RDONLY);
	if (fd != -1) ret_value = 1; else ret_value = 0;
	close(fd);
	
	return ret_value;
}

void VerificationPassword( void)
{
	CMD_GUI code=CMD_VERIFICATION_PASSWORD;
	// send the set contrast command
	write(gui_fd,&code,sizeof(code));
	read(gui_fd,&code,sizeof(code));	
	
}	

U16 Mp3_play(void)
{
	short code=CMD_MP3_PLAY;
	U16 ret_value;	
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&ret_value,sizeof(ret_value));
	return ret_value;
}
void Mp3_Record(void)
{
	short code=CMD_MP3_RECORD;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
void Mp3_Playback(void)
{
	short code=CMD_MP3_PLAYBACK;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}
void Mp3_play_init(char *pathname,U8 value,short effect)
{
	short code=CMD_MP3_INIT;
	// send the Turn on BackLight command
	ARG_LDA_MP3_INIT mp3_init_info={pathname,value,effect};
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&mp3_init_info,sizeof(mp3_init_info));
//	read(peri_fd,&code,sizeof(code));
}
void Mp3_play_close(void)
{
	short code=CMD_MP3_CLOSE;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));	
}
void sdcard_format(void)
{
	short code=CMD_SD_FORMAT;
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&code,sizeof(code));
}

U8 IsCardInsered(void)
{
	short code=CMD_SD_EXIST;
	U8 ret_value;	
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&ret_value,sizeof(ret_value));
	//printf("IsCardInsered..%d\n",ret_value);
	return ret_value;
}

int Mp3_get_play_count(void)
{
	short code=CMD_MP3_GET_POS;
	int ret_value;	
	// send the Turn on BackLight command
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&ret_value,sizeof(ret_value));
	return ret_value;
}	
	
void Set_volumne_effect(U8 value,short effect)
{
	short code=CMD_MP3_VOLUME_EFFECT;
	// send the Turn on BackLight command
	ARG_LDA_VOLUME_EFFECT volume_effect={value,effect};
	write(peri_fd,&code,sizeof(code));
	read(peri_fd,&volume_effect,sizeof(volume_effect));
}	