#ifndef _GUI_INTERNAL_H
#define _GUI_INTERNAL_H

#include "linux/gui.h"
#include "linux/mini_db.h"


#ifndef SEEK_SET
#define SEEK_SET	0
#endif
#ifndef SEEK_END
#define SEEK_END	2
#endif

/*************************************************************************
 *
 *	GUI's low level MARCOs 
 *
 *	Caution: these marcos will never be used by any external functions
 *
 *************************************************************************/
#define GUI_CAPTION_HEIGHT		14
#define GUI_CAPTION_X_SPACE		5
#define GUI_CAPTION_Y_SPACE		1
#define GUI_BUTTON_SPACE		2
#define GUI_FORM_BORDER_WIDTH	2
#define GUI_MENU_Y_SPACE		1
#define GUI_MENU_X_SPACE		4
#define GUI_MENUITEM_HORIZONTAL_LINE_HEIGHT	5
#define GUI_MENUITEM_HORIZONTAL_MAGIC_CHAR '-'

#define GUI_SPIN_SPACE			0
#define GUI_COMBOBOX_SPACE		2
#define GUI_SCROLLBAR_SPACE		2
#define GUI_CHECKBUTTON_SPACE	2
#define GUI_TEXTBOX_SPACE		1
#define GUI_CAPTION_SPACE		4

// time gap to generate a 'pen up' event
#define PEN_UP_TIME_OUT	5

// variables for calibration purpose ,defined in digi_public.c
extern int EDGE_X_MIN_LIMIT;
extern int EDGE_X_MAX_LIMIT;
extern int EDGE_Y_MIN_LIMIT;
extern int EDGE_Y_MAX_LIMIT;

extern int DIGI_X_MIN_LIMIT;
extern int DIGI_X_MAX_LIMIT;
extern int DIGI_Y_MIN_LIMIT;//0
extern int DIGI_Y_MAX_LIMIT;

extern S_MESSAGE m_queue[];

/*** define & initialize in peripheral.c *****/
extern SystemPreference SysPref;

/*** defined in k_default_msg_routine.c ***/
extern void (*UserMsgRoutine)(GUI_MSG *msg);
extern U32 user_msg_routine_a5;	// record the a5 world of user app.
extern short (*UserInputMethodRoutine)(U8 *s,short mac_char);
extern U32 user_input_method_a5;


/******* Battery warning message **********/
#ifdef CONFIG_PROCOM
	#define MSG_BATTERY_LOW_MSG	"Battery low,Please recharge it!"
#else
	#ifdef CONFIG_PALM3X
		#define MSG_BATTERY_LOW_MSG	"Battery low,Please change it!"
	#else
		#define MSG_BATTERY_LOW_MSG	"Battery low,Please change it!"
	#endif
	
#endif

#define MSG_BATTERY_LOW_TITLE	"Warning"


// mark if the hardware keyboard is enable
extern U8 KbStatus[];

extern short xBase,yBase; 
extern U8 * screen_ptr;
extern short screen_width;
extern short screen_height;

// the current available region 
extern RECT av_region;

// defined in k_mini_gui.c
extern FORM *active_frm;


// stretch the max contrast value to it!The default min is 0
#define MAX_CONTRAST_VALUE 0x100	

//AP JUMP TO  FLAG FOR SER ALARM 
#define APP_FLAG_END_YEAR 0x00
#define APP_FLAG_SCHEDULE 0x01
#define APP_FLAG_CLOCK 0x02
#define APP_FLAG_ALL 0xFF

//AP DEAL WITH FLAG FOR SER ALARM 
#define FLAG_END_YEAR 0x00
#define FLAG_ALARM 0x01

/*** control byte for PDA device peripherals ***/
#define CMD_SET_BACKLIGHT	0x01
#define CMD_GET_BATTERY		0x02
#define CMD_SET_CONTRAST		0x03
#define CMD_WRITE_SPEAKER	0x04
#define CMD_POWER_OFF		0x05
#define CMD_SET_AUTO_OFF	0x07
#define CMD_GET_AUTO_OFF	0x08
#define CMD_SUSPEND_MODE	0x09
#define CMD_IN_CHARGING	0x10
#define CMD_SET_DIGITIZER_DATA_TYPE 0x11		// SetDigitizerDataType
#define CMD_SET_SYS_BEEP_VOL	0x12
#define CMD_SET_ALARM_BEEP_VOL	0x13
#define CMD_SET_GAME_BEEP_VOL	0x14
#define CMD_GET_SYS_BEEP_VOL	0x15
#define CMD_GET_ALARM_BEEP_VOL	0x16
#define CMD_GET_GAME_BEEP_VOL	0x17
#define CMD_SYS_BEEP	0x18
#define CMD_GAME_BEEP	0x19
#define CMD_ALARM_BEEP	0x1a
#define CMD_SET_KB_TYPE 0x1b								// SetKbType
#define CMD_GET_MEM_STARTADDR	0x1c
#define CMD_RAMDISK_LOST	0x1d	// if the data in ramdisk lost
#define CMD_HARD_RESET	0x1e	// hard reset
#define CMD_SHOW_MEM		0x1f	// ShowMem
#define CMD_MP3_PLAY	0x20
#define CMD_DVR_RECORD	0x21
#define CMD_DVR_RECORD_PLAYBACK	0x22
#define CMD_MP3_INIT	0x23
#define CMD_MP3_CLOSE	0x24
#define CMD_MP3_GET_POS	0x25
#define CMD_SD_EXIST	0x26
#define CMD_SD_FORMAT	0x27
#define CMD_MP3_VOLUME_EFFECT	0x28
#define CMD_DVR_RECORD_INIT	0x29
#define CMD_DVR_RECORD_FINISHED	0x2a
#define CMD_DVR_PLAYBACK_INIT	0x2b
#define CMD_DVR_PLAYBACK_FINISHED	0x2c
#define CMD_DVR_PLAYBACK_GET_POS	0x2d
#define CMD_DVR_PLAYBACK_SET_POS	0x2e
#define CMD_DVR_PLAYBACK_SET_VOLUME	0x2f
#define CMD_RECORD_SD_INIT	0x30
#define CMD_SD_READ_BLOCK	0x31
#define CMD_SD_WRITE_BLOCK	0x32
#define CMD_SD_CARD_LED_DARK	0x33
#define CMD_SD_CARD_LED_BRIGHT	0x34
#define CMD_GET_RTC_TIME 0x35
#define CMD_SET_RTC_DAY  0x36
#define CMD_SET_RTC_HOUR 0x37
#define CMD_SET_ALARM 0x38
#define CMD_GET_RTC_HOUR 0x39
#define CMD_GET_RTC_DAY 0x3a

#define CMD_OPEN_FILE_IN_SD  0x40
#define CMD_CLOSE_FILE_IN_SD 0x41
#define CMD_WRITE_BLOCKS_IN_SD 0x42
#define CMD_READ_BLOCKS_IN_SD 0x43
#define CMD_CLOSE_SD_POWER 0x44
#define CMD_CHECK_VAR_DIR 0x45
#define CMD_OPEN_SD_POWER 0x46

#define CMD_DELETE_A_ALARM 0x50
#define CMD_UPDATA_ALARM 0x51
#define CMD_FIND_FIRST_ALARM 0x52
#define CMD_FIND_NEXT_ALARM 0x53
#define CMD_GET_ALARM_BY_PTR 0x54
#define CMD_SHOW_ALARM_MSG_BOX 0x55

#define CMD_MP3_POWER_OFF 0x60

#define CMD_WR_INODE_TO_ROOT 0x61
#define CMD_POWER_OFF_RECORDER 0x62
#define CMD_ENABLE_POWEROFF 0x63
#define CMD_DEL_FILE_IN_CARD 0x64
#define CMD_READ_ROOT_DIR 0x65
#define CMD_OPEN_FILE_IN_ROOT 0x66
#define CMD_READ_FROM_RANDOM_POS 0x67

#define CMD_DVR_VOLUME_IN_PLAYBACK 0x70

#define CMD_OLD_DVR_RECORD_SD_INIT    0x80
#define CMD_OLD_DVR_RECORD_INIT       0x81
#define CMD_OLD_DVR_RECORD_FINISHED   0x82
#define CMD_OLD_DVR_PLAYBACK_INIT	 0x83
#define CMD_OLD_DVR_PLAYBACK_FINISHED   0x84
#define CMD_OLD_DVR_PLAYBACK_GET_POS	0x85
#define CMD_OLD_DVR_PLAYBACK_SET_POS	0x86
#define CMD_OLD_DVR_PLAYBACK_SET_VOLUME 0x87
#define CMD_OLD_DVR_RECORD	0x88
#define CMD_OLD_DVR_RECORD_PLAYBACK	0x89
#define CMD_OLD_DVR_REPLAYBACK_INIT    0x8a

#define CMD_READ_CARD_CONTENT 0x90
#define CMD_FORMAT_UNKNOWN_FORMAT_CARD 0x91
#define CMD_GET_STRIP 0x95
#define CMD_GET_STRIP_STATUS 0x96
#define CMD_GET_RES_IN_INT 0x97
#define CMD_SET_TIMEOUT 0x99

typedef enum {
	CMD_CLIPRECT,
	CMD_SET_AVAILABLE_REGION,
	CMD_XOR_DRAW_LINE,		// xor_draw_line
	CMD_DRAW_LINE,				// draw_line
	CMD_DRAW_ELLIPSE,			// draw_ellipse
	CMD_DRAW_PIXEL,				// draw_pixel
	CMD_DRAW_RECT,				// draw_rect
	CMD_BITBLT,						// bitblt
	CMD_PATTERNFILL,				// patternfill
	CMD_CREATEFORM,				// CreateForm
	CMD_REFRESHCOMPONENT,	// RefreshComponent
	CMD_SETSPINBUTTONTEXT,	// SetSpinButtonText
	CMD_REFRESHMULTILINESTRING,	// RefreshMultiLineString
	CMD_REFRESHCOMBOBOXSTRING,		// RefreshComboBoxString
	CMD_HIDECOMPONENT,			// HideComponent
	CMD_CLOSEFORM,					// CloseForm
	CMD_SETFONT,						// SetFont	
	CMD_GETFONTHEIGHT,				// GetFontHeight
	CMD_GETSTRINGWIDTHINCLUDESPACING,	// GetStringWidthIncludeSpacing
	CMD_GETSTRINGWIDTH,				// GetStringWidth
	CMD_DRAWSTRINGINRECT,			// DrawStringInRect
	CMD_DO_DRAWSTRING,					// do_drawString
	CMD_DRAWSTRING,						// DrawString
	CMD_GETTEXTBOXLINES,				// GetTextBoxLines
	CMD_SETTEXTBOXCURSORPOS,		// SetTextBoxCursorPos
	CMD_REFRESHTEXTBOXCONTENTS,	// RefreshTextBoxContents	
	CMD_GETCURRENTACTIVATEDTEXTBOX,	// GetCurrentActivatedTextBox
	CMD_SETTEXT,										// set_text
	CMD_CLOSECURSOR,								// CloseCursor
	CMD_BLINKCURSOR,								// BlinkCursor
	CMD_SETFOCUS,									// SetFocus
	CMD_GETMESSAGE,								// GetMessage
	CMD_SHOWSOFTKEY,								// ShowSoftKey
	CMD_GUI_HOW_TO,								// gui_how_to
	CMD_GUI_ABOUT,									// gui_about
	CMD_SET_CALIBRATION,							// SetCalibration
	CMD_MSGBOX,											// MsgBox
	CMD_GET_REGISTER,							// GetRegister
	CMD_SET_REGISTER,							// SetRegister
	CMD_DISABLE_SYS_HARDWARE_KEY,			// DisableHardWareKey
	CMD_ENABLE_SYS_HARDWARE_KEY,			// EnableHardWareKey
	CMD_SET_CPU_SPEED,							// SetCPUSpeed
	CMD_GET_CPU_SPEED,							// GetCPUSpeed
	CMD_DEFAULT_MESSAGE_ROUTINE,		// DefaultMessageRoutine
	CMD_SEND_MESSAGE,							// SendMessage
	CMD_SHOW_SET_CONTRAST_FORM,			// ShowSetContrastForm
	CMD_GET_HANDHELD_TYPE,			// GetHandheldType
	CMD_SET_BAUD_RATE,					// SetBaudRate
	
	CMD_SET_USER_DEFINED_MSG_ROUTINE,		// SetUserDefinedMsgRoutine
	CMD_SET_USER_INPUT_METHOD,		// SetUserInputMethod
	
	CMD_PUT_APP_INFO,			// PutAppInfo
	CMD_GET_APP_INFO,			// GetAppInfo
	
	CMD_SET_INDICATOR,		// SetIndicator
	
	CMD_DISABLE_CURSOR,		// DisableCursor
	CMD_ENABLE_CURSOR,		// EnableCursor
	
	CMD_REGISTER_CH_FONTS,	// RegisterChFonts

	CMD_SET_CAPSLOCK,
	CMD_GET_CAPSLOCK,
	
	/**** the following is the temperary functions ******/
	CMD_DRAW_VSCROLLBUTTONS,	// draw_vscrollbuttons
	CMD_DRAW_CHECKBUTTON,		// draw_checkbutton
	CMD_DRAWMENU,						// draw_menu
	CMD_HIDE_MENUBAR,					// hide_menubar
	CMD_DEALWITH_TEXTBOX,				// dealwith_textbox
	/****add by cys ****/
	CMD_VERIFICATION_PASSWORD,
	CMD_REFRESH_LABEL_COMPONENT	// RefreshLabelComponent			
}CMD_GUI;

/**************************************************
 *
 *	Exported function argument descriptions
 *
 **************************************************/

// xor_draw_line(x1,y1,x2,y2);
typedef struct {
	short x1,y1,x2,y2;
}ARG_XOR_DRAW_LINE;

// draw_line(x1,y1,x2,y2,color)
typedef struct {
	short x1,y1,x2,y2,color;
}ARG_DRAW_LINE;

// draw_ellipse(x,y,a,b,*gc)
typedef struct {
	short x,y,a,b;
	GraphicsContext *gc;
}ARG_DRAW_ELLIPSE;

// draw_pixel(x,y,color)
typedef struct {
	short x,y;
	U8 color;
}ARG_DRAW_PIXEL;

// draw_rect(r,*gc)
typedef struct {
	RECT r;
	GraphicsContext *gc;
}ARG_DRAW_RECT;

// bitblt(src_x,src_y,w,h,dest_x,dest_y,*src,src_unit_per_line,*dest,dest_unit_per_line,*gc)
typedef struct {
	short src_x,src_y,w,h,dest_x,dest_y;
	U8 *src;
	short src_units_per_line;
	U8 *dest;
	short dest_units_per_line;
	GraphicsContext *gc;
}ARG_BITBLT;

// patternfill(dest_x,dest_y,w,h,*dest,dest_units_per_line,*gc)
typedef struct {
	short dest_x,dest_y,w,h;
	U8 *dest;
	short dest_units_per_line;
	GraphicsContext *gc;
}ARG_PATTERNFILL;

// RefreshComponent(idx,*dlg)
typedef struct {
	short idx;
	FORM *dlg;
}ARG_REFRESHCOMPONENT;

// RefreshComponent(idx,*dlg, y_offset_lines, max_lines)
typedef struct {
	short idx;
	FORM *dlg;
	short y_offset_lines;
	short max_lines;
}ARG_REFRESH_LABEL_COMPONENT;

// SetSpinButtonText(*dlg,*str)
typedef struct {
	FORM *dlg;
	char *str;
}ARG_SETSPINBUTTONTEXT;

// RefreshMultiLineString(*dlg,flag)
typedef struct  {
	FORM *dlg;
	short flag;
}ARG_REFRESHMULTILINESTRING;

// HideComponent(idx,*dlg)
typedef struct {
	short idx;
	FORM *dlg;
}ARG_HIDECOMPONENT;

// draw_checkbutton
typedef struct {
	FORM *dlg;
	short value;
}ARG_DRAW_CHECKBUTTON;

// GetStringWidthIncludeSpacing
typedef struct {
	char *str;
	short font_lib;
}ARG_GETSTRINGWIDTHINCLUDESPACING;

// GetStringWidth
typedef struct {
	char *str;
	short font_lib;
}ARG_GETSTRINGWIDTH;

// DrawStringInRect
typedef struct {
	RECT *r;
	char *data;
	GraphicsContext *gc;
	short font_lib,align_flag;
}ARG_DRAWSTRINGINRECT;

// do_drawString
typedef struct {
	short x,y;
	char *data;
	GraphicsContext *gc;
	short font_lib;
}ARG_DO_DRAWSTRING;

// DrawString
typedef struct {
	short x,y;
	char *str;
	GraphicsContext *gc;
}ARG_DRAWSTRING;

// GetTextBoxLines
typedef struct {
	FORM *dlg;
	short ret_value;
}ARG_GETTEXTBOXLINES;

// SetTextBoxCursorPos
typedef struct {
	FORM *dlg;
	short pos;
}ARG_SETTEXTBOXCURSORPOS;

// dealwith_textbox
typedef struct {
	FORM *dlg;
	PIXEL px;
}ARG_DEALWITH_TEXTBOX;

// RefreshTextBoxContnets
typedef struct {
	FORM *dlg;
	short start_pos,end_pos,flag;
}ARG_REFRESHTEXTBOXCONTENTS;
	
// set_text
typedef struct {
	FORM *dlg;
	U16 code;
}ARG_SETTEXT;

// SetFocus
typedef struct {
	FORM *dlg;
	short id;
}ARG_SETFOCUS;

// GetMessage(*dlg,*msg)
typedef struct {
	FORM *dlg;
	GUI_MSG *msg;
}ARG_GETMESSAGE;

// ShowSoftKey(*str,short max_char)
typedef struct {
	char *str;
	short max_char;
}ARG_SHOWSOFTKEY;

// gui_how_to(*str)
typedef struct {
	char *str;
	short ret_value;
}ARG_GUI_HOW_TO;

// gui_about(*title,*version)
typedef struct {
	char *title;
	char *version;
	short ret_value;
}ARG_GUI_ABOUT;

// SetCalibration(xmin,ymin,xmax,ymax)
typedef struct {
	int xmin;
	int ymin;
	int xmax;
	int ymax;
}ARG_SET_CALIBRATION;

// short MsgBox(char *msgstr,char *title,short mode)
typedef struct {
	char *msgstr;
	char *title;
	short mode;
	short ret_value;
}ARG_MSGBOX;

// short SetRegU8(U32 reg,U8 *data,short length) & GetRegU8(U32 reg,U8 *data,short length)
typedef struct {
	U32 reg;
	U32 *data;
	short length;
	short ret_value;
}ARG_RW_REGISTER;

// long SetCPUSpeed(U8,U8,U8)
typedef struct {
	U8 p;
	U8 q;
	U8 freq;
	U32 ret_value;
}ARG_SET_CPU_SPEED;

// long GetCPUSpeed(U8 *,U8 *,U8 *)
typedef struct {
	U8 *p;
	U8 *q;
	U8 *freq;
	U32 ret_value;
}ARG_GET_CPU_SPEED;

// DefaultMessageRoutine(FORM *,GUI_MSG *)
typedef struct {
	FORM *dlg;
	GUI_MSG *msg;
}ARG_DEFAULT_MESSAGE_ROUTINE;

// SetUserMsgRoutine(*fn,a5_world)
typedef struct {
	void *fn;
	U32 a5;
}ARG_CALLBACK_FUNCTION;

// SetIndicator(FORM *form,short status)
typedef struct {
	FORM *frm;
	short status;
}ARG_SET_INDICATOR;

/***** mini db related commands ****/
typedef enum {
	CMD_LDA_CREATE_DB,	//LDACreateDB
	CMD_LDA_ADD_DB_CATEGORY,	// LDAAddDBCategory
	CMD_LDA_DELETE_DB_CATEGORY, // LDADeleteDBCategory
	CMD_LDA_UPDATE_DB_CATEGORY,	// LDAUpdateDBCategory
	CMD_LDA_GET_DB_CATEGORY_TOTAL,	// LDAGetDBCategoryTotal
	CMD_LDA_GET_DB_CATEGORY_ALPHABET,	// LDAGetDBCategoryAlphbet
	CMD_LDA_OPEN_DB,	// LDAOpenDB
	CMD_LDA_DELETE_DB,	// LDADeleteDB
	CMD_LDA_CLOSE_DB,	// LDACloseDB
	CMD_LDA_SET_DB_CATEGORY,	// LDASetDBCategory
	CMD_LDA_SET_DB_SECURITY,	// LDASetDBSecurity
	CMD_LDA_GET_RECORD_NUMBER,	// LDAGetRecordNumber
	CMD_LDA_GET_TOTAL_RECORDS,	// LDAGetTotalRecords
	CMD_LDA_ADD_RECORD,	// LDAAddRecord
	CMD_LDA_FIND_RECORD_BY_VALUE,	// LDAFindRecordByValue
	CMD_LDA_GET_NEXT_RECORD,	// LDAGetNextRecord
	CMD_LDA_GET_RECORD,	// LDAGetRecord
	CMD_LDA_DELETE_RECORD,	// LDADeleteRecord
	CMD_LDA_UPDATE_RECORD,	// LDAUpdateRecord
	CMD_LDA_CONVERT_BYTE,	// LDAConvertByte
	CMD_MY_LDA_GET_RECORD,	// my_LDAGetRecord, added by david for GEhealth
}CMD_MINI_DB;	

// LDACreateDB
typedef struct {
	char *DBName;
	short DBFlag;
	DBFIELD *dbField;
	short dbFieldNumber;
	int ret_value;
}ARG_LDA_CREATE_DB;

// LDAAddDbCategory
typedef struct {
	DBHANDLE fdidx;
	char * cate_name;
	int ret_value;
}ARG_LDA_ADD_DB_CATEGORY;

// LDADeleteDbCategory
typedef struct {
	DBHANDLE fdidx;
	int cate_index;
	int ret_value;
}ARG_LDA_DELETE_DB_CATEGORY;

// LDAUpdateCategory
typedef struct {
	DBHANDLE fdidx;
	int cate_index;
	char *cate_name;
	int ret_value;
}ARG_LDA_UPDATE_DB_CATEGORY;

// LDAGetDbCaterotyTotal
typedef struct {
	DBHANDLE fdidx;
	int ret_value;
}ARG_LDA_GET_DB_CATEGORY_TOTAL;

typedef struct {
	DBHANDLE fdidx;
	int cate_rank;
	char * cate_name;
	int ret_value;
}ARG_LDA_GET_DB_CATEGORY_ALPHABET;

typedef struct {
	char *DBName;
	short DBFlag;
	DBHANDLE ret_value;
}ARG_LDA_OPEN_DB;

typedef struct {
	char *DBName;
	int ret_value;
}ARG_LDA_DELETE_DB;

typedef struct {
	DBHANDLE fdidx;
	short DBFlag;
	int ret_value;
}ARG_LDA_CLOSE_DB;

typedef struct {
	DBHANDLE fdidx;
	char cate_option;
	int ret_value;
}ARG_LDA_SET_DB_CATEGORY;

typedef struct {
	DBHANDLE fdidx;
	char secu_option;
	int ret_value;
}ARG_LDA_SET_DB_SECURITY;

typedef struct {
	DBHANDLE fdidx;
	int ret_value;
}ARG_LDA_GET_RECORD_NUMBER;

typedef struct {
	DBHANDLE fdidx;
	int ret_value;
}ARG_LDA_GET_TOTAL_RECORDS;

typedef struct {
	DBHANDLE fdidx;
	void *data;
	char category;
	char property;
	int ret_value;
}ARG_LDA_ADD_RECORD;

typedef struct {
	DBHANDLE fdidx;
	void *data;
	void * CompareValue;
	char * category;
	char * property;
	int ret_value;
}ARG_LDA_FIND_RECORD_BY_VALUE;

typedef struct {
	DBHANDLE fdidx;
	void *data;
	long RecordId;
	char * category;
	char * property;
	int ret_value;
}ARG_LDA_GET_RECORD;

typedef struct {
	DBHANDLE fdidx;
	void *data;
	long RecordId;
	int flag;
	int ret_value;
}ARG_MY_LDA_GET_RECORD;

typedef struct {
	DBHANDLE fdidx;
	void *data;
	char *category;
	char *property;
	int ret_value;
}ARG_LDA_GET_NEXT_RECORD;

typedef struct {
	DBHANDLE fdidx;
	short RecordId;
	int ret_value;
}ARG_LDA_DELETE_RECORD;

typedef struct {
	DBHANDLE fdidx;
	void *data;
	short RecordId;
	char category;
	char property;
	int ret_value;
}ARG_LDA_UPDATE_RECORD;


typedef struct {
	char *DBName;
	short option;
	int ret_value;
}ARG_LDA_CONVERT_BYTE;

typedef struct {
	char pathname[100];
	U8 value;
	short effect;
	short ret_value;	
}ARG_LDA_MP3_INIT;

typedef struct {
	U8 value;
	short effect;	
}ARG_LDA_VOLUME_EFFECT;

typedef struct {
	char record_name[16];
	int start_block;
	int block_num;
	char reserve[8];
}Index_Of_Record;

typedef struct {
	int Last_Loca_Block_id;
	int Record_num;
	int Sd_Capacity;
	char Flag[14];
	char reserve[6];
}Location_Pos;

typedef struct {
	char * pathname;
	U8 value;
	U8 mode;
	int current_dvr_pos;
	int ret_value;
}ARG_LDA_PLAYBACK_INIT;

typedef struct {
	Index_Of_Record * idx;
	U8 value;
}ARG_OLD_LDA_PLAYBACK_INIT;

typedef struct {
	Index_Of_Record * idx;
	U8 value;
}ARG_OLD_LDA_REPLAYBACK_INIT;

typedef struct {
	unsigned char hour;
	unsigned char min;
}ARG_LDA_SET_RTC_HOUR;

typedef struct {
	short year;
	short yday;
}ARG_LDA_SET_RTC_DAY;

typedef struct {
	short year;
	short mon;
	short day;
	short yday;
	short hour;
	short min;
	short sec;	
	short fnflag;
	short appflag;		
	int ret_value;
	char Description[50];
	U8 Repeat_event;
	U8 Repeat_type;
}ARG_LDA_SET_ALARM;

typedef struct alarm_time {
	short year;
	short mon;
	short day;
	short yday;
	short hour;
	short min;
	short days;
	short sec;
	short fnflag;
	short appflag;
	char Description[50];
	U8 Repeat_event;
	U8 Repeat_type;				
	struct alarm_time * next;
}STUCT_ALARM_TIME;

typedef struct {
	short year;
	short yday;
	short hour;
	short min;
}ARG_DELETE_A_ALARM;

typedef struct {
	short year;
	short yday;
	short hour;
	short min;
	char Description[50];
	U8 Repeat_event;
	U8 Repeat_type;
}ARG_UPDATA_ALARM;

typedef struct {
	ARG_LDA_SET_ALARM *First_Alarm;
	short appflag;
	int *alarm_ptr;	
	unsigned short ret_val;
}ARG_FIND_FIRST_ALARM;

typedef struct {
	ARG_LDA_SET_ALARM *Next_Alarm;
	short appflag;
	int *alarm_ptr;
	unsigned short ret_val;
}ARG_FIND_NEXT_ALARM;

typedef struct {
	int alarm_ptr;
	ARG_LDA_SET_ALARM *Alarm_Rec;
	unsigned short ret_val;
}ARG_GET_ALARM_BY_PTR;

typedef struct {
	char * pathname;
	int mode;
	int file_counter;
	int ret_val;
}ARG_OPEN_FILE_IN_SD;

typedef struct {
	int mode;
	char * src_name;
	int ret_val;
}ARG_CLOSE_FILE_IN_SD;

typedef struct {
	int block;
	char *buff;
	U8 ret_value;
}ARG_LDA_SD_READ_BLOCK;

typedef struct {
	int block;
	char *buff;
	U8 ret_value;
}ARG_LDA_SD_WRITE_BLOCK;

typedef struct {
	char * buffer;
	unsigned short len;
	unsigned long counter;
	int ret_val;
}ARG_WRITE_BLOCKS_IN_SD;

typedef struct {
	char * buffer;
	unsigned short len;
	unsigned short ret_val;
}ARG_READ_BLOCKS_IN_SD;

typedef struct {
	int mode;
	int ret_val;
}ARG_OPEN_SD_POWER;

typedef struct {
	int mode;
	int changed;
	int ret_val;
}ARG_CLOSE_SD_POWER;

#define CMD_READ_ROOT_DIR 0x65
typedef struct {
	int index;
	char * file_name;
	unsigned long * p_file_size;
	int mode;
	int ret_val;
}ARG_READ_ROOT_DIR;

typedef struct{
	char * file_name;
	int ret_val;
}ARG_WR_INODE_TO_ROOT;

typedef struct {
	char * pathname;
	int mode;
	int ret_val;
}ARG_DEL_FILE_IN_CARD;

typedef struct {
	char * pathname;
	unsigned long * file_length;
	unsigned long * first_cluster;
	int ret_val;
} ARG_OPEN_FILE_IN_ROOT;

typedef struct {
	unsigned long first_cluster;
	unsigned long file_length;
	unsigned long pos;
	unsigned long len;
	unsigned char * buf;
	int ret_val;
} ARG_READ_FROM_RANDOM_POS;
#endif